import "package:flutter/material.dart";
import 'dart:convert';
// import 'dart:io';
import 'package:http/http.dart' as http;

class SearchBox extends StatefulWidget {
  final apiKey;
  final corpusId;
  final customerId;
  final image;
  final results;
  final callback;

  SearchBox(
      {Key? key,
      this.apiKey,
      this.corpusId,
      this.customerId,
      this.results,
      this.image,
      this.callback})
      : super(key: key);

  @override
  _SearchBox createState() => new _SearchBox();
}

class _SearchBox extends State<SearchBox>
    with SingleTickerProviderStateMixin {
  static TextEditingController _controller = TextEditingController();
  FocusNode _focus = new FocusNode();
  bool isFocused = false;
  String fieldValue = "";
  static String apiKey = "";
  static String customerId = "";
  static List corpusId = [];
  static int results = 0;
  static late AnimationController _animationController;
  static late Animation<double> _animation;

  @override
  void dispose() {
    super.dispose();
  }

  @override
  void initState() {
    _animationController =
        AnimationController(vsync: this, duration: Duration(seconds: 1));
    _animationController.repeat(reverse: true);
    _animation = Tween(begin: 1.0, end: 0.5).animate(_animationController)
      ..addListener(() {
        setState(() {});
      });
    _animationController.reset();
    super.initState();

    _controller = TextEditingController();
    _focus.addListener(_onFocusChange);
    apiKey = widget.apiKey;
    customerId = widget.customerId;
    results = widget.results > 0 ? widget.results : 10;
    for (int i = 0; i < widget.corpusId.length; i++) {
      corpusId.add(widget.corpusId[i]);
    }
  }

  void _onFocusChange() {
    setState(() {
      isFocused = _focus.hasFocus;
    });
    debugPrint("Focus: " + _focus.hasFocus.toString());
  }

  Future handleQuery() async {
    final res = await runSearch(fieldValue);
    widget.callback(res);
  }

  static runSearch(String query) async {
    _animationController.repeat();
    _controller = TextEditingController(text: query);
    final List corpusKey = [];
    for (int i = 0; i < corpusId.length; i++) {
      corpusKey.add({"customer_id": customerId, "corpus_id": corpusId[i]});
    }
    print(corpusKey);
    final uri = Uri.parse("https://h.serving.zir.dev:443/v1/query");
    final body = jsonEncode({
      "query": [
        {"query": query, "num_results": results , "corpus_key": corpusKey}
      ]
    });

    // http request from android
    // HttpClient httpClient = new HttpClient();
    // HttpClientRequest request = await httpClient.postUrl(uri);
    // request.headers.set("Content-Type", "application/json");
    // request.headers.set("x-api-key", apiKey);
    // request.headers.set("customer-id", customerId);
    // request.add(utf8.encode(body));
    // HttpClientResponse response = await request.close();
    // String reply = await response.transform(utf8.decoder).join();
    // httpClient.close();

    final reply = await http.post(uri,
        headers: {
          "Content-Type": "application/json",
          "x-api-key": apiKey,
          "customer-id": customerId,
        },
        body: body);

    _animationController.reset();
    // return reply;
    return reply.body;
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      width: 400,
      child: Card(
          child: Material(
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(5.0),
        ),
        elevation: _focus.hasFocus ? 10 : 0,
        child: Padding(
          padding: EdgeInsets.only(left: 5.0, right: 10.0),
          child: Row(
            children: [
              Expanded(
                child: Padding(
                  padding: EdgeInsets.only(right: 5.0),
                  child: TextField(
                      focusNode: _focus,
                      decoration: InputDecoration(
                        border: InputBorder.none,
                        focusedBorder: InputBorder.none,
                      ),
                      controller: _controller,
                      onChanged: (String value) {
                        fieldValue = value;
                      },
                      onSubmitted: (String value) {
                        handleQuery();
                      }),
                ),
              ),
              MaterialButton(
                disabledElevation: 0,
                hoverColor: Colors.transparent,
                splashColor: Colors.transparent,
                focusColor: Colors.transparent,
                height: 26,
                minWidth: 26,
                padding: EdgeInsets.all(0),
                onPressed: () {
                  handleQuery();
                },
                child: Opacity(
                  opacity: _animation.value,
                  child: widget.image != "" 
                  ? Image.asset(widget.image, height: 26, width: 26,) 
                  : Image(
                    height: 26,
                    width: 26,
                    image: AssetImage("assets/logo.png",
                        package: "search_box"
                        )
                  ),
                ),
              )
            ],
          ),
        ),
      )),
    );
  }
}
